##########################################
######## INTRUMENTAL VARIABLES ###########
##########################################

library(AER)
data("CigarettesSW")
summary(CigarettesSW)
help(CigarettesSW)


# Does the taxation on cigarettes reduces the cigarettes consumtpion?
# More precisely, how much taxes must be increased to reach a certain 
# reduction in cigarette consumption? 

# Economists use elasticities to answer this kind of question. 
# Since the price elasticity for the demand of cigarettes is unknown, it must be estimated. 

#An OLS regression of log quantity on log price cannot be used to estimate the effect of 
#interest since there is simultaneous causality between demand and supply. 

#SOLUTION: Intrumental variables

#We use the data set CigarettesSW which comes with the package AER. 
#It is a panel data set that contains observations on cigarette consumption and several 
#economic indicators for all 48 continental federal states of the U.S. in 1985 and 1995.
#(We consider data for the cross section of states in 1995 only).




# compute real per capita prices
CigarettesSW$rprice <- with(CigarettesSW, price / cpi)

#  compute the sales tax
CigarettesSW$salestax <- with(CigarettesSW, (taxs - tax) / cpi)

# check the correlation between sales tax and price
cor(CigarettesSW$salestax, CigarettesSW$rprice)
#> [1] 0.6141228

# generate a subset for the year 1995
c1995 <- subset(CigarettesSW, year == "1995")


### log(c1995$packs)= a+ b*log(rprice) + e
### Z<-salestax

##################################
##################################
##################################
   ####  FIRST STAGE ####
##################################
##################################
##################################

# perform the first stage regression
cig_s1 <- lm(log(rprice) ~ salestax, data = c1995)
coeftest(cig_s1, vcov = vcovHC, type = "HC1")
#inspect the R^2 of the first stage regression
summary(cig_s1)$r.squared

# store the predicted values
lcigp_pred <- cig_s1$fitted.values

##################################
##################################
##################################
####  SECOND STAGE ####
##################################
##################################
##################################

# run the stage 2 regression
cig_s2 <- lm(log(c1995$packs) ~ lcigp_pred)
coeftest(cig_s2, vcov = vcovHC)

# perform TSLS using 'ivreg()'
cig_ivreg <- ivreg(log(packs) ~ log(rprice) | salestax, data = c1995)
coeftest(cig_ivreg, vcov = vcovHC, type = "HC1")


#COMMENTS

# Running the individual regressions for each stage of TSLS using lm() leads to the same 
#coefficient estimates as when using ivreg(). 

#However, the standard errors reported for the second-stage regression, e.g.,
#by coeftest() or summary(), are invalid: neither adjusts for using predictions from 
#the first-stage regression as regressors in the second-stage regression. 

# Fortunately, ivreg() performs the necessary adjustment automatically. 
# This is another advantage over manual step-by-step estimation which we have done above 
#for demonstrating the mechanics of the procedure.

# An increase in cigarette prices by one percent reduces cigarette consumption by roughly 1.08
# percentage points, which is fairly elastic. 
# Attention: keep in mind that this estimate might be incorrect even though we used IV 
# estimation: there still might be a bias due to omitted variables.


# While this is hard to answer for general IV regression, in the case of a single endogenous
# regressor X one may use the following rule of thumb:



#Thus a multiple IV regression approach is needed.

# Consider the case of a single endogenous regressor. If the coefficients on all instruments 
# in the population first-stage regression of a TSLS estimation are zero, the instruments 
# do not explain any of the variation in the X. 
# Although the latter case is unlikely to be encountered in practice, we should ask 
# ourselves “to what extent” the assumption of instrument relevance should be fulfilled.

# Compute the F-statistic which corresponds to the hypothesis that the coefficients on are
# all zero in the first-stage regression. If the F-statistic<10, the 
# instruments are weak such that the TSLS estimate of the coefficient on X is biased and 
# no valid statistical inference about its true value can be made. 



c1995 <- subset(CigarettesSW, year == "1995")

# add cigtax to the data set
CigarettesSW$cigtax <- with(CigarettesSW, tax/cpi)

c1995 <- subset(CigarettesSW, year == "1995")



s1<-lm(log(rprice)~ salestax + cigtax, data = c1995)
summary(s1) # F-statistic>10
s2 <- lm(log(packs) ~s1$fitted.values, data = c1995)


# estimate the model
cig_ivreg3 <- ivreg(log(packs) ~ log(rprice) | 
                        salestax + cigtax, data = c1995)

coeftest(cig_ivreg3, vcov = vcovHC, type = "HC1")




