library(plm)
data("Grunfeld", package="plm")
# A balanced panel of 10 observational units 
# (firms) from 1935 to 1954
# Small N, high T!
head(Grunfeld)

#### EQUIVALENT ESTIMATES ####
ols<-lm(inv~value+capital, data=Grunfeld)
pols<-plm(inv~value+capital, data=Grunfeld, model = "pooling",index =c("firm", "year"))
summary(ols)
summary(pols)

ols$coefficients==pols$coefficients


##Check if there are individual effects:

pFtest(inv~value+capital, data = Grunfeld)

# H0: no individual effects -> OLS via individual effect models
# p-value < 2.2e-16 : THERE ARE INDIVIDUAL EFFECTS:
# How we control for individual effects for inv~value+capital ?

#Within Estimator:

within <-plm(inv~value+capital, data=Grunfeld, model = "within",index =c("firm", "year"))
summary(within)
#Within estimator also allows for estimating individual effectss!
fixef(within)


#First Difference Estimator:

fd <-plm(inv~value+capital, data=Grunfeld, model = "fd",index =c("firm", "year"))
summary(fd)

#Between Estimator:

between<-plm(inv~value+capital, data=Grunfeld, model = "between",index =c("firm", "year"))
summary(between)

#Random Effect Estimator:
random<-plm(inv~value+capital, data=Grunfeld, model = "random",index =c("firm", "year"))
summary(random)

# What is the best one?
# Compare these models via Hausman Test
phtest(inv~value+capital, data=Grunfeld)
# Individual effect are uncorrelated with the regressors:
# Random effect conditions are true!!!

### Wait...what about the time effect? 
# Lagrange Multiplier Test - (Breusch-Pagan) for balanced panels
plmtest(inv ~ value + capital, effect=c("time"), type=("bp"), data = Grunfeld,index=c("firm", "year"))
#p-value < 0.01107


#Within Estimator controlling for both individual and time effect:

within.time <-plm(inv~value+capital+factor(year), data=Grunfeld, model = "within",index =c("firm", "year"))
summary(within.time)
#Within estimator also allows for estimating individual effectss!
fixef(within.time)

random.time <-plm(inv~value+capital+factor(year), data=Grunfeld, model = "random",index =c("firm", "year"))
summary(random.time)



#Now, what else?

#Check the assumptions on the error terms!

######### Testing for serial correlation ############

#Breusch–Godfrey Test for Panel Models

# Serial correlation tests apply to macro panels with long time series. 
# Not a problem in micro panels (with very few years). 
# H0: there is not serial correlation.

pbgtest(within) #H0 rejected: serial correlation in errors
pbgtest(fd) #H0 rejected: serial correlation in errors
pbgtest(between) # why it turns out an error?
# We are testing for serial correlation of the errors:
# We want to check if errors for the subject i are correlated in time.
# But the Between estimator....
pbgtest(random) #H0 rejected: serial correlation in errors
pbgtest(within.time) #H1 rejected: no serial correlation 
pbgtest(random.time) #H0 rejected: serial correlation in errors

# H0 rejected, there is serial correlation in our panel data.

######### Testing for heteroscedasticity ############
# Breusch-Pagan test against heteroskedasticity

library(lmtest)
bptest(within)
bptest(fd)
bptest(between) 
bptest(random) 
bptest(within.time)
bptest(random.time)

# In case of Heteroscedasticity:

# with vcovHC we get Heteroscedasticity-Consistent Covariance Matrix Estimation: 
coeftest(within, vcov. = vcovHC(within,sandwich=TRUE))
coeftest(fd, vcov. = vcovHC)
coeftest(between, vcov. = vcovHC) 
coeftest(random, vcov. = vcovHC) 
coeftest(within.time, vcov. = vcovHC) 
coeftest(random.time, vcov. = vcovHC) 



data("Gasoline", package="plm")
#A panel of 18 observations from 1960 to 1978
head(Gasoline)
str(Gasoline)

# lgaspcar: logarithm of motor gasoline consumption per car

# lincomep: logarithm of real per-capita income

# lrpmg: logarithm of real motor gasoline price

# lcarpcap: logarithm of the stock of cars per capita

#### EQUIVALENT ESTIMATES ####
ols<-lm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline)
pols<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline,index =c("country", "year"), model = "pooling")
summary(ols)
summary(pols)


##Check if there are individual effects:

pFtest(inv~value+capital, data = Grunfeld)

# H0: no individual effects -> OLS via individual effect models
# p-value < 2.2e-16 : THERE ARE INDIVIDUAL EFFECTS.

#Within Estimator:
within<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline,index =c("country", "year"), model = "within")
summary(within)
#Within estimator also allows for estimating individual effects!
fixef(within)
#First Difference Estimator:
fd<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline,index =c("country", "year"), model = "fd")
summary(fd)
#Between Estimator:
between<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline,index =c("country", "year"), model = "between")
summary(between)
#Random Effect Estimator:
random<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline,index =c("country", "year"), model = "random")
summary(random)
# What is the best one?
# Compare these models via Hausman Test
phtest(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap, data=Gasoline)
# Individual effect are correlated with the regressors:
# Fixed effect assumption holds!!!

# Consistent estimators when FE assumption holds are:
# within estimator
# First differences estimator

#Within Estimator:
within.time<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap+factor(year), data=Gasoline,index =c("country", "year"), model = "within")
summary(within.time)
#Within estimator also allows for estimating individual effects!
fixef(within.time)
#First Difference Estimator:
fd.time<-plm(lgaspcar~lincomep+lrpmg+lrpmg+lcarpcap+factor(year), data=Gasoline,index =c("country", "year"), model = "fd")
summary(fd.time)

#So, what is the better model for this data? 

#Now, what else?

#Check the assumptions on the error terms!

######### Testing for serial correlation ############

#Breusch–Godfrey Test for Panel Models
pbgtest(within)
pbgtest(within.time)
pbgtest(fd)

# H0 rejected, there is serial correlation in our panel data.
pbgtest(fd.time)
# H1 rejected, no serial correlation in our panel data.

######### Testing for heteroscedasticity ############
# Breusch-Pagan test against heteroskedasticity

library(lmtest)
bptest(within)
bptest(fd)
bptest(within.time)
bptest(fd.time)
# H1 rejected: homoscedastic errors.

coeftest(within, vcov. = vcovHC(within.time,sandwitch=TRUE))
coeftest(within.time, vcov. = vcovHC(within.time,sandwitch=TRUE))
coeftest(fd, vcov. = vcovHC(within.time,sandwitch=TRUE))
summary(fd.time)




